@@ -52,11 +52,11 @@
 #define SCIM_PROP_INPUT_MODE_LATIN           "/IMEngine/Anthy/InputMode/Latin"
 #define SCIM_PROP_INPUT_MODE_WIDE_LATIN      "/IMEngine/Anthy/InputMode/WideLatin"
 
-#define SCIM_PROP_CONV_MODE                  "/IMEngine/ANthy/ConvMode"
-#define SCIM_PROP_CONV_MODE_MULTI_SEG        "/IMEngine/ANthy/ConvMode/MultiSegment"
-#define SCIM_PROP_CONV_MODE_SINGLE_SEG       "/IMEngine/ANthy/ConvMode/SingleSegment"
-#define SCIM_PROP_CONV_MODE_MULTI_REAL_TIME  "/IMEngine/ANthy/ConvMode/MultiRealTime"
-#define SCIM_PROP_CONV_MODE_SINGLE_REAL_TIME "/IMEngine/ANthy/ConvMode/SingleRealTime"
+#define SCIM_PROP_CONV_MODE                  "/IMEngine/Anthy/ConvMode"
+#define SCIM_PROP_CONV_MODE_MULTI_SEG        "/IMEngine/Anthy/ConvMode/MultiSegment"
+#define SCIM_PROP_CONV_MODE_SINGLE_SEG       "/IMEngine/Anthy/ConvMode/SingleSegment"
+#define SCIM_PROP_CONV_MODE_MULTI_REAL_TIME  "/IMEngine/Anthy/ConvMode/MultiRealTime"
+#define SCIM_PROP_CONV_MODE_SINGLE_REAL_TIME "/IMEngine/Anthy/ConvMode/SingleRealTime"
 
 #define SCIM_PROP_TYPING_METHOD              "/IMEngine/Anthy/TypingMethod"
 #define SCIM_PROP_TYPING_METHOD_ROMAJI       "/IMEngine/Anthy/TypingMethod/RomaKana"
@@ -179,15 +179,21 @@ AnthyInstance::process_key_event_input (const KeyEvent &key)
 bool
 AnthyInstance::process_key_event_lookup_keybind (const KeyEvent& key)
 {
+    m_last_key = key;
+
     std::vector<Action>::iterator it;
-    for (it = m_factory->m_actions.begin();
+    for (it  = m_factory->m_actions.begin();
          it != m_factory->m_actions.end();
          it++)
     {
-        if (it->perform (this, key))
+        if (it->perform (this, key)) {
+            m_last_key = KeyEvent ();
             return true;
+        }
     }
 
+    m_last_key = KeyEvent ();
+
     return false;
 }
 
@@ -262,7 +268,7 @@ AnthyInstance::process_key_event (const KeyEvent& key)
 
     // for wide Latin mode
     if (m_preedit.get_input_mode () == SCIM_ANTHY_MODE_WIDE_LATIN)
-        process_key_event_wide_latin_mode (key);
+        return process_key_event_wide_latin_mode (key);
 
     // for other mode
     if (get_typing_method () != SCIM_ANTHY_TYPING_METHOD_NICOLA ||
@@ -1011,12 +1017,18 @@ AnthyInstance::action_insert_space (void)
         is_wide = true;
     }
 
-    if (is_wide)
+    if (is_wide) {
         commit_string (utf8_mbstowcs ("\xE3\x80\x80"));
-    else
+        return true;
+    } else if (get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA || // FIXME! it's a ad-hoc solution.
+               (m_last_key.code != SCIM_KEY_space &&
+                m_last_key.code != SCIM_KEY_KP_Space))
+    {
         commit_string (utf8_mbstowcs (" "));
+        return true;
+    }
 
-    return true;
+    return false;
 }
 
 bool 
@@ -1040,12 +1052,18 @@ AnthyInstance::action_insert_alternative_space (void)
         is_wide = true;
     }
 
-    if (is_wide)
+    if (is_wide) {
         commit_string (utf8_mbstowcs ("\xE3\x80\x80"));
-    else
+        return true;
+    } else if (get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA || // FIXME! it's a ad-hoc solution.
+               (m_last_key.code != SCIM_KEY_space &&
+                m_last_key.code != SCIM_KEY_KP_Space))
+    {
         commit_string (utf8_mbstowcs (" "));
+        return true;
+    }
 
-    return true;
+    return false;
 }
 
 bool
@@ -1054,9 +1072,14 @@ AnthyInstance::action_insert_half_space (void)
     if (m_preedit.is_preediting ())
         return false;
 
-    commit_string (utf8_mbstowcs (" "));
+    if (m_last_key.code != SCIM_KEY_space &&
+        m_last_key.code != SCIM_KEY_KP_Space)
+    {
+        commit_string (utf8_mbstowcs (" "));
+        return true;
+    }
 
-    return true;
+    return false;
 }
 
 bool
@@ -1976,10 +1999,8 @@ AnthyInstance::process_helper_event (const String &helper_uuid,
             m_preedit.convert (selection);
             set_preedition ();
             set_lookup_table ();
-        }
-
-        if (cursor >= (int) len &&
-            surround.substr (cursor - len, len) == selection)
+        } else if (cursor >= (int) len &&
+                   surround.substr (cursor - len, len) == selection)
         {
             delete_surrounding_text (0 - len, len);
             m_preedit.convert (selection);
