@@ -203,14 +203,6 @@ KanaConvertor::append (const KeyEvent & key,
     return false;
 }
 
-bool
-KanaConvertor::append (const String   & str,
-                       WideString     & result,
-                       WideString     & pending)
-{
-    return false;
-}
-
 void
 KanaConvertor::clear (void)
 {
@@ -236,24 +228,12 @@ KanaConvertor::flush_pending (void)
 }
 
 void
-KanaConvertor::set_case_sensitive (bool sens)
-{
-}
-
-bool
-KanaConvertor::get_case_sensitive (void)
-{
-    return false;
-}
-
-void
-KanaConvertor::set_pending (String str)
+KanaConvertor::reset_pending (const WideString &result, const String &raw)
 {
     m_pending = String ();
-    if (has_voiced_consonant (str))
-        m_pending = str;
+    if (has_voiced_consonant (utf8_wcstombs (result)))
+        m_pending = utf8_wcstombs (result);
 }
-
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
